
from rdflib import URIRef, Literal, Graph
from .prefixes import NIF, XSD, ITSRDF, RDF
from .prefixes import NIFPrefixes

from six import binary_type

class NIFPhrase(object):
    """
    Represents an annotation in a document.
    """
    
    def __init__(self,
            context = None,
            annotator = None,
            mention = None,
            beginIndex = None,
            endIndex = None,
            score = None,
            taIdentRef = None,
            taClassRef = None,
            taMsClassRef = None,
            uri = None,
            is_hash_based_uri = False,
            source = None):
        """
        A phrase can be represented by an OffsetBasedString URI or a
        ContextHashBasedString URI.

        :pram: is_hash_based_uri set to True indicates that the :param: uri 
        is a ContextHashBasedString, otherwsie is considered as OffsetBasedString.

        ContextHashBasedString is discussed in 
        the paper Linked-Data Aware URI Schemes for Referencing Text Fragments 
        (https://doi.org/10.1007/978-3-642-33876-2_17) page 4. 
        The ContextHashBasedString URI must be provided by the users, it is not
        created automatically.
        """
        self.context = context
        self.annotator = annotator
        self.mention = mention
        self.beginIndex = beginIndex
        self.endIndex = endIndex
        self.score = score
        self.taIdentRef = taIdentRef
        self.taClassRef = taClassRef
        self.taMsClassRef = taMsClassRef
        self.isContextHashBasedString = is_hash_based_uri
        self.original_uri = uri
        self.source = source
        
    @property
    def uri(self):
        return URIRef(self.original_uri or self.generated_uri)
        
    @property
    def generated_uri(self):
        return self.context.split('#')[0] + '#offset_' + str(self.beginIndex) + '_' + str(self.endIndex)

    def triples(self):
        """
        Returns the representation of the phrase as RDF triples
        """
        if self.isContextHashBasedString:
            yield (self.uri, RDF.type, NIF.ContextHashBasedString)
        else:
            yield (self.uri, RDF.type, NIF.OffsetBasedString)
        yield (self.uri, RDF.type, NIF.Phrase)
        yield (self.uri, NIF.anchorOf, Literal(self.mention))
        yield (self.uri, NIF.beginIndex, Literal(self.beginIndex, datatype=XSD.nonNegativeInteger))
        yield (self.uri, NIF.endIndex, Literal(self.endIndex, datatype=XSD.nonNegativeInteger))

        if self.annotator is not None:
            yield (self.uri, ITSRDF.taAnnotatorsRef, URIRef(self.annotator))
        if self.score is not None:
            yield (self.uri, ITSRDF.taConfidence, Literal(str(float(self.score)), datatype=XSD.double, normalize=False))
        if self.taIdentRef is not None:
            yield (self.uri, ITSRDF.taIdentRef, URIRef(self.taIdentRef))
        if self.taMsClassRef is not None:
            yield (self.uri, NIF.taMsClassRef, URIRef(self.taMsClassRef))
        for currentClassRef in self.taClassRef or []:
            yield (self.uri, ITSRDF.taClassRef, URIRef(currentClassRef))
        if self.context is not None:
            yield (self.uri, NIF.referenceContext, URIRef(self.context))
        if self.taMsClassRef is not None:
            yield (self.uri, NIF.taMsClassRef, URIRef(self.taMsClassRef))
        if self.source is not None:
            yield (self.uri, ITSRDF.taSource, Literal(self.source, datatype=XSD.string))
            
    @classmethod
    def load_from_graph(cls, graph, uri):
        """
        Given a RDF graph and a URI which represents a phrase in
        that graph, load the corresponding phrase.
        """
        phrase = cls()
        phrase.original_uri = uri
        for s,p,o in graph.triples((uri, None, None)):
            if p == NIF.anchorOf:
                phrase.mention = o.toPython()
            elif p == NIF.beginIndex:
                phrase.beginIndex = o.toPython()
            elif p == NIF.endIndex:
                phrase.endIndex = o.toPython()
            elif p == NIF.referenceContext:
                phrase.context = o.toPython()
            elif p == ITSRDF.taAnnotatorsRef:
                phrase.annotator = o.toPython()
            elif p == ITSRDF.taConfidence:
                phrase.score = o.toPython()
            elif p == ITSRDF.taIdentRef:
                phrase.taIdentRef = o.toPython()
            elif p == NIF.taMsClassRef:
                phrase.taMsClassRef = o.toPython()
            elif p == ITSRDF.taClassRef:
                if phrase.taClassRef is None:
                    phrase.taClassRef = []
                phrase.taClassRef.append(o.toPython())
            elif p == ITSRDF.taSource:
                phrase.source = o.toPython()
            if o == NIF.ContextHashBasedString :
                phrase.isContextHashBasedString = True
        return phrase

    @property
    def turtle(self):
        graph = Graph()
        for triple in self.triples():
            graph.add(triple)
        
        graph.namespace_manager = NIFPrefixes().manager
        out = graph.serialize(format='turtle')
        
        # workaround for https://github.com/RDFLib/rdflib/issues/884
        if isinstance(out, binary_type):
            out = out.decode('utf-8')
        return out

    def __str__(self):
        return self.__repr__()
    
    def __repr__(self):
        if (self.mention is not None
            and self.beginIndex is not None
            and self.endIndex is not None):
            mention = self.mention
            if len(mention) > 50:
                mention = mention[:50]+'...'
            return '<NIFPhrase {}-{}: {}>'.format(self.beginIndex, self.endIndex, repr(mention))
        else:
            return '<NIFPhrase (undefined)>'
        
    def _tuple(self):
        return (self.context,
        self.annotator,
        self.mention,
        self.beginIndex,
        self.endIndex,
        self.score,
        self.taIdentRef,
        set(self.taClassRef) if self.taClassRef else set(),
        self.taMsClassRef,
        self.uri,
        self.source)
        
    def __eq__(self, other):
        return self._tuple() == other._tuple()
    
    def __hash__(self):
        return hash(self.uri)




import unittest
from pynif.phrase import NIFPhrase
from .util import turtle_equal
from rdflib import Graph, URIRef

class NIFPhraseTest(unittest.TestCase):
    
    sample_ttl = """
        @prefix xsd:   <http://www.w3.org/2001/XMLSchema#> .
        @prefix itsrdf: <http://www.w3.org/2005/11/its/rdf#> .
        @prefix nif:   <http://persistence.uni-leipzig.org/nlp2rdf/ontologies/nif-core#> .
                
        <http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc/yn_08Oct08_file_0/#offset_91_102>
            a                       nif:OffsetBasedString , nif:Phrase ;
            nif:anchorOf            "Afghanistan" ;
            nif:beginIndex          "91"^^xsd:nonNegativeInteger ;
            nif:endIndex            "102"^^xsd:nonNegativeInteger ;
            nif:referenceContext    <http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc/yn_08Oct08_file_0/#offset_0_1413> ;
            itsrdf:taConfidence     "1"^^xsd:double ;
            itsrdf:taIdentRef       <http://dbpedia.org/resource/Afghanistan> .
        """
    
    example_text = "    Primary Navigation Secondary Navigation Search: Nearly 60 militants killed in southern Afghanistan Tue Oct 7, 9:14 AM ET KABUL (Reuters) - U.S.-led coalition and Afghan security forces killed nearly 60 militants during separate clashes in southern Afghanistan, the U.S. military and a police official said Tuesday. Violence has surged in the war-torn country with some 3,800 people, a third of them civilians, killed as a result of the conflict by the end of July this year, according to the United Nations. U.S.-led coalition and Afghan security forces killed 43 militants during heavy fighting in Qalat district of southern Zabul province Sunday, the U.S. military said in a statement Tuesday. \"ANSF (Afghan National Security Forces ) and coalition forces on a patrol received heavy weapons, machine gun and sniper fire from militants in multiple locations,\" the U.S. military said in a statement. The combined forces responded with small arms fire , rocket propelled grenades and close air support , killing the militants, it said. No Afghan or U.S.-led troops were killed or wounded during incident, it said. In a separate incident, Afghan and international troops killed 16 Taliban insurgents and wounded six more during a gun battle in Nad Ali district of southern Helmand province on Monday, provincial police chief Asadullah Sherzad told Reuters. (Writing by Jonathon Burch; Editing by Bill Tarrant)"
    example_turtle_ContextHashBasedString = """
        @prefix xsd:   <http://www.w3.org/2001/XMLSchema#> .
        @prefix itsrdf: <http://www.w3.org/2005/11/its/rdf#> .
        @prefix nif:   <http://persistence.uni-leipzig.org/nlp2rdf/ontologies/nif-core#> .
                
        <http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc#hash_1400_1411_9f215ab78483a223463f9726ee0e92c0_%20%20%20%20Primary%20Navigati>
            a                       nif:ContextHashBasedString , nif:Phrase ;
            nif:anchorOf            "Afghanistan" ;
            nif:beginIndex          "91"^^xsd:nonNegativeInteger ;
            nif:endIndex            "102"^^xsd:nonNegativeInteger ;
            nif:referenceContext    <http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc#hash_0_1411_6218664a3a8c7bed58460e329ddc6904_%20%20%20%20Primary%20Navigati> ;
            itsrdf:taConfidence     "1"^^xsd:double ;
            itsrdf:taIdentRef       <http://dbpedia.org/resource/Afghanistan> .
        """

    def test_to_string_blank(self):
        b = NIFPhrase()
        self.assertEqual("<NIFPhrase (undefined)>", str(b))
        
    def test_to_string(self):
        b = NIFPhrase()
        b.beginIndex = 34
        b.endIndex = 44
        b.mention = "revolution"
        self.assertEqual("""<NIFPhrase 34-44: 'revolution'>""", str(b))
        
    def test_original_uri(self):
        b = NIFPhrase()
        b.original_uri = 'http://example.com/my_annotation'
        self.assertEqual('http://example.com/my_annotation', str(b.uri))
        
    def test_to_turtle(self):
        b = NIFPhrase()
        b.context = 'http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc/yn_08Oct08_file_0/#offset_0_1413'
        b.mention = 'Afghanistan'
        b.beginIndex = 91
        b.endIndex = 102
        b.taClassRef = None
        b.score = 1
        b.taIdentRef = 'http://dbpedia.org/resource/Afghanistan'
        
        self.assertTrue(turtle_equal(self.sample_ttl, b.turtle))
        
    def test_parse(self):
        g = Graph().parse(format='turtle',data=self.sample_ttl)
        uri = URIRef('http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc/yn_08Oct08_file_0/#offset_91_102')
        phrase = NIFPhrase.load_from_graph(g, uri)
        
        self.assertEqual('Afghanistan', phrase.mention)
        self.assertEqual(91, phrase.beginIndex)
        self.assertEqual(102, phrase.endIndex)
        self.assertEqual(1., phrase.score)
        self.assertEqual('http://dbpedia.org/resource/Afghanistan', phrase.taIdentRef)

    def test_create_ContextHashBasedString_phrase(self):
        phrase = NIFPhrase(
            uri='http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc#hash_1400_1411_9f215ab78483a223463f9726ee0e92c0_%20%20%20%20Primary%20Navigati',
            is_hash_based_uri = True,
            context = "http://www.cse.iitb.ac.in/~soumen/doc/CSAW/doc#hash_0_1411_6218664a3a8c7bed58460e329ddc6904_%20%20%20%20Primary%20Navigati",
            beginIndex=91,
            endIndex=102,
            mention= self.example_text[91:102],
            score=1.,
            taIdentRef="http://dbpedia.org/resource/Afghanistan")

        self.assertEqual('Afghanistan', phrase.mention)
        self.assertEqual(91, phrase.beginIndex)
        self.assertEqual(102, phrase.endIndex)
        self.assertEqual(1., phrase.score)    
        self.assertTrue(turtle_equal(self.example_turtle_ContextHashBasedString, phrase.turtle))

if __name__ == '__main__':
    unittest.main()


import unittest
import os
from pynif.collection import NIFCollection
from .util import turtle_equal

class NIFCollectionTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        testdir = os.path.dirname(os.path.abspath(__file__))
        with open(os.path.join(testdir, 'data/reuters-nif20.ttl'), 'r') as f:
            cls.example_nif20 = f.read()
        with open(os.path.join(testdir, 'data/reuters-nif20-normalized.ttl'), 'r') as f:
            cls.example_nif20_normalized = f.read()
        with open(os.path.join(testdir, 'data/reuters-nif21.ttl'), 'r') as f:
            cls.example_nif21 = f.read()
        with open(os.path.join(testdir, 'data/example-maradona.ttl'), 'r') as f:
            cls.example_maradona = f.read()
        cls.testdir = testdir

    def test_serialize_nif21(self):
        collection = NIFCollection.loads(self.example_nif21)

        self.assertEqual(1, len(collection.contexts))
        context = collection.contexts[0]
        self.assertEqual(3, len(context.phrases))

        in_nif = collection.dumps()

        self.assertTrue(turtle_equal(self.example_nif21, in_nif))

    def test_serialize_nif20(self):
        collection = NIFCollection.loads(self.example_nif20)

        self.assertEqual(1, len(collection.contexts))
        context = collection.contexts[0]
        self.assertEqual(3, len(context.phrases))

        in_nif = collection.dumps()

        self.assertTrue(turtle_equal(self.example_nif20_normalized, in_nif))

    def test_file_helpers(self):
        c = NIFCollection.load(os.path.join(self.testdir, 'data/reuters-nif21.ttl'))
        c2 = NIFCollection.loads(self.example_nif21)
        self.assertEqual(c, c2)

    def test_create(self):
        collection = NIFCollection(uri="http://freme-project.eu")

        context = collection.add_context(
            uri="http://freme-project.eu/doc32",
            mention="Diego Maradona is from Argentina.")

        context.add_phrase(
           beginIndex=0,
           endIndex=14,
           taClassRef=['http://dbpedia.org/ontology/SportsManager', 'http://dbpedia.org/ontology/Person', 'http://nerd.eurecom.fr/ontology#Person'],
           score=0.9869,
           annotator='http://freme-project.eu/tools/freme-ner',
           taIdentRef='http://dbpedia.org/resource/Diego_Maradona',
           taMsClassRef='http://dbpedia.org/ontology/SoccerManager')

        context.add_phrase(
           beginIndex=23,
           endIndex=32,
           taClassRef=['http://dbpedia.org/ontology/PopulatedPlace', 'http://nerd.eurecom.fr/ontology#Location',
            'http://dbpedia.org/ontology/Place'],
           score=0.9804,
           annotator='http://freme-project.eu/tools/freme-ner',
           taMsClassRef='http://dbpedia.org/resource/Argentina')

        generated_nif = collection.dumps(format='turtle')
        self.assertTrue(turtle_equal(self.example_maradona, generated_nif))

        parsed_collection = NIFCollection.loads(generated_nif)
        parsed_context = parsed_collection.contexts[0]

        self.assertEqual(1, len(parsed_collection.contexts))
        self.assertEqual(collection, parsed_collection)


if __name__ == '__main__':
    unittest.main()


from rdflib import URIRef, Literal, Graph
from .phrase import NIFPhrase
from .prefixes import NIF, XSD, ITSRDF, RDF, DCTERMS, nif_ontology_uri
from .prefixes import NIFPrefixes

from six import binary_type


class NIFContext(object):
    """
    A context is a string which can be annotated by phrases.
    """

    def __init__(self,
                 beginIndex=None,
                 endIndex=None,
                 mention=None,
                 sourceUrl=None,
                 uri=None,
                 is_hash_based_uri = False):
        """
        A context can be represented by an OffsetBasedString URI or a
        ContextHashBasedString URI.

        :pram: is_hash_based_uri set to True indicates that the :param: uri 
        is a ContextHashBasedString, otherwsie is considered as OffsetBasedString.

        ContextHashBasedString is discussed in 
        the paper Linked-Data Aware URI Schemes for Referencing Text Fragments 
        (https://doi.org/10.1007/978-3-642-33876-2_17) page 4. 
        The ContextHashBasedString URI must be provided by the users, it is not
        created automatically.
        """
        self.isContextHashBasedString = is_hash_based_uri
        self.original_uri = uri
        self.beginIndex = beginIndex
        self.endIndex = endIndex
        self.mention = mention
        if mention is not None and beginIndex is None:
            self.beginIndex = 0
        if mention is not None and endIndex is None:
            self.endIndex = len(mention)
        self.sourceUrl = sourceUrl
        self.phrases = []

    def add_phrase(self,
            beginIndex=None,
            endIndex=None,    
            annotator = None,
            score = None,
            taIdentRef = None,
            taClassRef = None,
            taMsClassRef = None,
            uri = None,
            source = None,
            is_hash_based_uri = False):
        """
        Creates a new annotation in this document.
        
        :returns: the new {@class NIFPhrase}
        """
        phrase = NIFPhrase(context = self.original_uri,
                beginIndex = beginIndex,
                endIndex = endIndex,
                annotator = annotator,
                score = score,
                taIdentRef = taIdentRef,
                taClassRef = taClassRef,
                taMsClassRef = taMsClassRef,
                uri = uri,
                source = source,
                is_hash_based_uri= is_hash_based_uri)
        if beginIndex is not None and endIndex is not None:
            phrase.mention = self.mention[beginIndex:endIndex]
        self.phrases.append(phrase)
        return phrase
    
    @property
    def uri(self):
        return URIRef(self.original_uri)
    
    def triples(self):
        """
        Returns the representation of the context as RDF triples
        """
        if self.isContextHashBasedString:
            yield (self.uri, RDF.type, NIF.ContextHashBasedString)
        else:
            yield (self.uri, RDF.type, NIF.OffsetBasedString)
        yield (self.uri, RDF.type, NIF.Context)
        yield (self.uri, NIF.beginIndex, Literal(self.beginIndex, datatype=XSD.nonNegativeInteger))
        yield (self.uri, NIF.endIndex, Literal(self.endIndex, datatype=XSD.nonNegativeInteger))
        yield (self.uri, NIF.isString, Literal(self.mention))
        if self.sourceUrl is not None:
            yield (self.uri, NIF.sourceUrl, URIRef(self.sourceUrl))
               
        for phrase in self.phrases:
            for triple in phrase.triples():
                yield triple
        
    @classmethod
    def load_from_graph(cls, graph, uri):
        """
        Given a RDF graph and a URI which represents a context in
        that graph, load the corresponding context and its child phrases.
        """
        context = cls()
        context.original_uri = uri.toPython()
        # Load core data
        for s,p,o in graph.triples((uri, None, None)):
            if p == NIF.isString:
                context.mention = o.toPython()
            elif p == NIF.beginIndex:
                context.beginIndex = o.toPython()
            elif p == NIF.endIndex:
                context.endIndex = o.toPython()
            elif p == NIF.sourceUrl:
                context.sourceUrl = o.toPython()
            if o == NIF.ContextHashBasedString :
                context.isContextHashBasedString = True
            
        # Load child phrases
        for s,p,o in graph.triples((None, NIF.referenceContext, uri)):
             phrase = NIFPhrase.load_from_graph(graph, s)
             context.phrases.append(phrase)
             
        return context

    @property
    def turtle(self):
        graph = Graph()
        for triple in self.triples():
            graph.add(triple)
        
        graph.namespace_manager = NIFPrefixes().manager
        out = graph.serialize(format='turtle')
        
        # workaround for https://github.com/RDFLib/rdflib/issues/884
        if isinstance(out, binary_type):
            out = out.decode('utf-8')
        return out
    
    def __str__(self):
        return self.__repr__()
    
    def __repr__(self):
        if (self.mention is not None
            and self.beginIndex is not None
            and self.endIndex is not None):
            mention = self.mention
            if len(mention) > 50:
                mention = mention[:50]+'...'
            return '<NIFContext {}-{}: {}>'.format(self.beginIndex, self.endIndex, repr(mention))
        else:
            return '<NIFContext (undefined)>'
        
    def _tuple(self):
        return (self.uri,
            self.beginIndex,
            self.endIndex,
            self.mention,
            self.sourceUrl,
            set(self.phrases))
    
    def __eq__(self, other):
        return self._tuple() == other._tuple()
    
    def __hash__(self):
        return hash(self.uri)
    



from rdflib.graph import Graph
from rdflib.compare import isomorphic
from rdflib.compare import graph_diff

def turtle_equal(a, b):
    """
    Given two strings representing turtle-encoded RDF,
    check whether they represent the same graph.
    """
    ga = Graph().parse(format='turtle',data=a)
    for x, y, z in ga:
        print((x,y,z))
    gb = Graph().parse(format='turtle',data=b)
    eq = isomorphic(ga, gb)
    if not eq:
        both, first, second = graph_diff(ga, gb)
        print("Present in both:")
        print(both)
        print("Present in first:")
        print(first)
        print("Present in second:")
        print(second)
    return eq



from .context import NIFContext
from rdflib import Graph, URIRef
from .prefixes import RDF, NIF, DCTERMS, NIFPrefixes, nif_ontology_uri
from six import binary_type

class NIFCollection(object):
    """
    A collection (or context collection) is a set of contexts (snippets of texts that
    are annotated by phrases).
    """

    def __init__(self, uri=None):
        self.contexts = []
        self.original_uri = uri

    @property
    def uri(self):
        return URIRef(self.original_uri)

    def add_context(self, uri, mention, beginIndex=None, endIndex=None):
        """
        Adds a context to the collection. Returns the context.
        """
        c = NIFContext(uri=uri, mention=mention, beginIndex=beginIndex, endIndex=endIndex)
        self.contexts.append(c)
        return c

    def triples(self):
        """
        Generates all the triples used to represent this collection.
        """
        if self.original_uri is not None:
            yield (self.uri, RDF.type, NIF.ContextCollection)
            yield (self.uri, DCTERMS.conformsTo, URIRef(nif_ontology_uri))
            for context in self.contexts:
                yield (self.uri, NIF.hasContext, context.uri)

        for context in self.contexts:
            for triple in context.triples():
                yield triple

    @classmethod
    def load_from_graph(cls, graph, uri=None):
        """
        Given a RDF graph, load all the contexts it contains
        as one collection object.

        If no URI is provided, all the nif:Context in the graph
        will be loaded (compatibility with NIF 2.0).
        """
        uri_ref = None
        if uri is not None:
            uri_ref = uri.toPython()

        collection = NIFCollection(uri=uri_ref)
        # Load collection
        if uri is not None:
            context_uris = [o for s,p,o in graph.triples((uri, NIF.hasContext, None))]
        else:
            context_uris = [s for s,p,o in graph.triples((None, RDF.type, NIF.Context))]

        for u in context_uris:
            context = NIFContext.load_from_graph(graph, u)
            collection.contexts.append(context)

        return collection

    @classmethod
    def loads(cls, data, format='turtle', uri=None):
        """
        Load a collection from a string representation of an RDF graph.

        If no URI for the collection is provided, we scan the graph to find
        the first ContextCollection in it. In there isn't any, the collection
        will load all available nif:Context in the graph.
        """
        g = Graph().parse(format=format,data=data)
        if uri is None:
            for s,p,o in g.triples((None, RDF.type, NIF.ContextCollection)):
                uri = s
                break
        return cls.load_from_graph(g, uri)

    @classmethod
    def load(cls, filename, format='turtle', uri=None):
        """
        Load a collection from a file designated by its filename.

        If no URI for the collection is provided, we scan the graph to find
        the first ContextCollection in it. In there isn't any, the collection
        will load all available nif:Context in the graph.
        """
        with open(filename, 'r') as f:
            return cls.loads(f.read(), format=format, uri=uri)

    def dumps(self, format='turtle'):
        """
        Returns a string representation of the graph in RDF.
        """
        graph = Graph()
        for triple in self.triples():
            graph.add(triple)

        graph.namespace_manager = NIFPrefixes().manager
        out = graph.serialize(format=format)

        # workaround for https://github.com/RDFLib/rdflib/issues/884
        if isinstance(out, binary_type):
            out = out.decode('utf-8')

        return out

    def dump(self, filename, format='turtle'):
        """
        Dumps the collection to a file.
        """
        with open(filename, 'w') as f:
            f.write(self.dumps(format=format))

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return '<NIFCollection {}>'.format(self.uri)

    def __eq__(self, other):
         return (self.uri == other.uri and set(self.contexts) == set(other.contexts))



name = "pynif"

from .phrase import NIFPhrase
from .context import NIFContext
from .collection import NIFCollection
from .prefixes import NIFPrefixes

__all__ = [ 'NIFPhrase', 'NIFContext', 'NIFCollection', 'NIFPrefixes' ]
